#include "mqtt_iot02.h"
#include "ui_mqtt_iot02.h"
#include <QDebug>
#include <QJsonParseError>

Mqtt_IoT02::Mqtt_IoT02(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::Mqtt_IoT02)
{
    ui->setupUi(this);

    mqttBasic = new MqttBasic;
    indicadorLed = new IndicadorLed;
    indicadorBotons = new IndicadorGrupBotonsEt;
    ui->hlAmunt->addWidget(mqttBasic);
    ui->hlCentre->addWidget(indicadorLed);
    ui->hlAvall->addWidget(indicadorBotons);
    connect(mqttBasic,SIGNAL(vSignalMqttConnected()),this,SLOT(vMqttConnectat()));
    connect(mqttBasic,SIGNAL(vSignalMqttDisconnected()),this,SLOT(vMqttDesconnectat()));
    connect(indicadorLed,SIGNAL(vTrametJsonLeds(QString)),this,SLOT(vTrametJsonLeds(QString)));

    indicadorBotons->vSetJsonText("");

    //showMaximized();
}

Mqtt_IoT02::~Mqtt_IoT02()
{
    delete ui;
}

void Mqtt_IoT02::vMqttConnectat(){
    indicadorLed->vSetEnableCheckBoxes(true);
    qsTemaBtIO0 = "/"+mqttBasic->qsMac()+"/btIO0";
    mqttBasic->vTopicSubscription(qsTemaBtIO0);
    qsTemaBtI34 = "/"+mqttBasic->qsMac()+"/btI34";
    mqttBasic->vTopicSubscription(qsTemaBtI34);
    qsTemaBtI35 = "/"+mqttBasic->qsMac()+"/btI35";
    mqttBasic->vTopicSubscription(qsTemaBtI35);
    qsTemaJsonInReq = "/"+mqttBasic->qsMac()+"/jsonInReq";
    qsTemaJsonIn = "/"+mqttBasic->qsMac()+"/jsonIn";
    mqttBasic->vTopicSubscription(qsTemaJsonIn);

    connect(mqttBasic,SIGNAL(vSignalMqttMsgRcv(QString,QString)),
            this,SLOT(vSlotMqttMsgRcv(QString,QString)));
    timer = new QTimer(this);
    connect(timer,SIGNAL(timeout()),this,SLOT(vCridaTemporitzador()));
    timer->start(500);
}

void Mqtt_IoT02::vMqttDesconnectat(){
    //qDebug() << "MQTT desconnectat!";
    indicadorLed->vSetEnableCheckBoxes(false);
    disconnect(mqttBasic,SIGNAL(vSignalMqttMsgRcv(QString,QString)));
    timer->stop();
    disconnect(timer,SIGNAL(timeout()));
}

void Mqtt_IoT02::vTrametJsonLeds(QString qs){
    QString qsTema = "/"+mqttBasic->qsMac()+"/jsonLeds";
    qDebug() << "Tema: "<<qsTema<<" Missatge: "<<qs;
    mqttBasic->vPublish(qsTema,qs);
}

void Mqtt_IoT02::vSlotMqttMsgRcv(QString qsT,QString qsM){
    if(qsT == qsTemaBtIO0){
        //qDebug() << "Botó IO0: " << qsM;
        indicadorBotons->vSetEstatBoto("btIO0",qsM.endsWith("pressed"));
    }
    if(qsT == qsTemaBtI34){
        //qDebug() << "Botó I34: " << qsM;
        indicadorBotons->vSetEstatBoto("btI34",qsM.endsWith("pressed"));
    }
    if(qsT == qsTemaBtI35){
        //qDebug() << "Botó I35: " << qsM;
        indicadorBotons->vSetEstatBoto("btI35",qsM.endsWith("pressed"));
    }
    if(qsT == qsTemaJsonIn){
        indicadorBotons->vSetJsonText(qsM);

        qDebug() << "LDR: " << QString(tr("%1").arg(nLdrDesDeCadenaTextJson(qsM)));
    }
}

void Mqtt_IoT02::vCridaTemporitzador(){
//    static int n = 0;

//    qDebug() << QString(tr("%1").arg(++n));
    mqttBasic->vPublish(qsTemaJsonInReq,"Demanant JSON d'entrades");
}

int Mqtt_IoT02::nLdrDesDeCadenaTextJson(QString qsJson){
    QByteArray qba = qsJson.toLatin1();

    QJsonParseError parseError;
    QJsonDocument jsonDoc;
    jsonDoc = QJsonDocument::fromJson(qba, &parseError);
    if(parseError.error != QJsonParseError::NoError){
        qWarning() << "Parse error at " << parseError.offset << ":" << parseError.errorString();
        return -1;
    }
    QJsonObject jsonObj = jsonDoc.object();
    QJsonValue subobjLDR = jsonObj["LDR"];

    return subobjLDR.toInt();
}
