#include "mqttbasic.h"
#include "ui_mqttbasic.h"
#include <QDateTime>
#include <QDebug>
#include <QMessageBox>

MqttBasic::MqttBasic(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::MqttBasic)
{
    ui->setupUi(this);

    m_client = new QMqttClient(this);
    m_client->setHostname(ui->leBroker->text());
    m_client->setPort(MQTT_PORT);
    if(ui->leUser->text()!="" && ui->lePwd->text()!=""){
        m_client->setUsername(ui->leUser->text());
        m_client->setPassword(ui->lePwd->text());
    }

    connect(m_client, &QMqttClient::stateChanged, this, &MqttBasic::updateLogStateChange);
    connect(m_client, &QMqttClient::disconnected, this, &MqttBasic::brokerDisconnected);

    connect(m_client, &QMqttClient::messageReceived, this, [this](const QByteArray &message, const QMqttTopicName &topic) {
        const QString content = QDateTime::currentDateTime().toString()
                    + QLatin1String(" Received Topic: ")
                    + topic.name()
                    + QLatin1String(" Message: ")
                    + message
                    + QLatin1Char('\n');
        emit vSignalMqttMsgRcv(topic.name(),message);
        qDebug() << content;
    });
}

MqttBasic::~MqttBasic()
{
    if (m_client->state() == QMqttClient::Connected)
        m_client->disconnectFromHost();
    delete ui;
}

void MqttBasic::on_btConnect_clicked(){
    if (m_client->state() == QMqttClient::Disconnected) {
        ui->leBroker->setEnabled(false);
        ui->leUser->setEnabled(false);
        ui->lePwd->setEnabled(false);
        ui->leMAC->setEnabled(false);
        ui->btConnect->setText(tr("Dis&connect"));
        m_client->setHostname(ui->leBroker->text());
        m_client->setPort(MQTT_PORT);
        if(ui->leUser->text()!="" && ui->lePwd->text()!=""){
            m_client->setUsername(ui->leUser->text());
            m_client->setPassword(ui->lePwd->text());
        }
        m_client->connectToHost();
    } else {
        brokerDisconnected();

        m_client->disconnectFromHost();
    }
}

void MqttBasic::updateLogStateChange(){
    const QString content = QDateTime::currentDateTime().toString()
                    + QLatin1String(": State Change ")
                    + QString::number(m_client->state())
                    + QLatin1Char('\n');

    if(QString::number(m_client->state()) == "2"){
        ui->lbMqttInfo->setStyleSheet("QLabel { color : green; }");
        ui->lbMqttInfo->setText("Connected");
        emit vSignalMqttConnected();
    }
    qDebug() << content;
}

void MqttBasic::brokerDisconnected(){
    ui->leBroker->setEnabled(true);
    ui->leUser->setEnabled(true);
    ui->lePwd->setEnabled(true);
    ui->leMAC->setEnabled(true);
    ui->btConnect->setText(tr("&Connect"));
    ui->lbMqttInfo->setStyleSheet("QLabel { color : red; }");
    ui->lbMqttInfo->setText("Disconnected");
    emit vSignalMqttDisconnected();
}

void MqttBasic::vTopicSubscription(QString qs){
    auto subscription = m_client->subscribe(qs);
    if (!subscription) {
        QMessageBox::critical(this, QLatin1String("Error"), QLatin1String("Could not subscribe. Is there a valid connection?"));
        return;
    }
}

void MqttBasic::vPublish(QString qsTopic,QString qsMsg){
    if (m_client->publish(qsTopic, qsMsg.toUtf8()) == -1)
        QMessageBox::critical(this, QLatin1String("Error"), QLatin1String("Could not publish message"));
}

QString MqttBasic::qsMac(){
    return ui->leMAC->text();
}
